<?php

namespace Drupal\monarch_csv_import;

/**
 * Iterates over csv rows.
 */
class CsvIterator implements \Iterator {

  /**
   * Holds the current data.
   *
   * @var array|null
   */
  protected $data;

  /**
   * Holds the header columns.
   *
   * @var array
   */
  protected $header;

  /**
   * Holds the file stream.
   *
   * @var array
   */
  protected $file;

  /**
   * Index for the data.
   *
   * @var int
   */
  protected $index;

  /**
   * {@inheritdoc}
   */
  public function __construct(string $file_path) {
    if (!($this->file = fopen($file_path, 'r'))) {
      throw new \Exception('Unable to open ' . $file_path . ' for reading.');
    }

    $this->rewind();
  }

  /**
   * {@inheritdoc}
   */
  public function __destruct() {
    if ($this->file) {
      fclose($this->file);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function current() {
    return $this->data;
  }

  /**
   * {@inheritdoc}
   */
  public function key() {
    return $this->index;
  }

  /**
   * {@inheritdoc}
   */
  public function next() : void {
    if ($this->data) {
      $this->index++;
    }

    $fields = fgetcsv($this->file) ?: NULL;
    $data = [];

    if (!empty($fields)) {
      foreach ($fields as $index => $field) {
        if (!isset($this->header[$index]) || empty($this->header[$index])) {
          throw new \Exception('Missing column name for column ' . ($index + 1));
        }

        $data[$this->header[$index]] = $field;
      }

      $this->data = $data;
    }
    else {
      $this->data = NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function rewind() : void {
    fseek($this->file, 0);
    $this->header = fgetcsv($this->file);
    $this->index = 0;
    $fields = fgetcsv($this->file) ?: NULL;
    $data = [];

    if (!empty($fields)) {
      foreach ($fields as $index => $field) {
        if (!isset($this->header[$index])) {
          throw new \Exception('Missing column for column ' . ($index + 1));
        }

        $data[$this->header[$index]] = $field;
      }

      $this->data = $data;
    }
    else {
      $this->data = NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function valid() : bool {
    return !empty($this->data);
  }

  /**
   * Returns all of the column names in the csv.
   *
   * @return array
   *   The list of column names.
   */
  public function columns() : array {
    return $this->header;
  }

}
